/*
 * LicenceAmendDialog.java
 *
 * Created on October 20, 2001, 2:13 PM
 */

package fina2.bank;

import java.awt.GridBagConstraints;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.transaction.UserTransaction;

import fina2.Main;
import fina2.ui.UIManager;
import fina2.ui.table.TableRow;
import fina2.ui.tree.Node;

/**
 * 
 * @author Kakha Kakhidze
 * @changed Vaso Pirtskhalaishvili
 */
public class LicenceAmendDialog extends javax.swing.JDialog {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;
	private fina2.Main main = fina2.Main.main;

	private LicenceTypeSelectDialog licenceTypeDialog;

	private Node node;
	private Licence licence;
	private LicenceTypePK licenceTypePK;
	private LicencePK licencePK;
	private BankPK pk;
	private boolean canAmend;
	private boolean ok;

	public LicenceAmendDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);

		ui.loadIcon("fina2.cancel", "cancel.gif");
		ui.loadIcon("fina2.help", "help.gif");
		ui.loadIcon("fina2.ok", "ok.gif");
		ui.loadIcon("fina2.find", "find.gif");

		licenceTypeDialog = new LicenceTypeSelectDialog(parent, true);
		initComponents();
	}

	public boolean isOk() {
		return ok;
	}

	public Node getNode() {
		return node;
	}

	public void show(Node node, BankPK pk, boolean canAmend) {
		this.node = node;
		this.pk = pk;
		this.canAmend = canAmend;

		selLicenceTypeButton.setEnabled(canAmend);
		licenceCodeText.setEditable(canAmend);
		licenceDateText.setEditable(canAmend);
		licenceDateOfChangeText.setEditable(canAmend);
		licenceReasonText.setEditable(canAmend);
		operationalCheckBox.setEnabled(canAmend);

		if (node != null) {

			licencePK = (LicencePK) node.getPrimaryKey();

			try {
				InitialContext jndi = fina2.Main.getJndiContext();

				Object ref = jndi.lookup("fina2/bank/Licence");
				LicenceHome home = (LicenceHome) PortableRemoteObject.narrow(ref, LicenceHome.class);

				licence = home.findByPrimaryKey(licencePK);

				ref = jndi.lookup("fina2/bank/LicenceType");

				LicenceTypeHome licenceTypeHome = (LicenceTypeHome) PortableRemoteObject.narrow(ref, LicenceTypeHome.class);
				licenceTypePK = licence.getTypePK();
				LicenceType licenceType = licenceTypeHome.findByPrimaryKey(licenceTypePK);

				licenceTypeText.setText(licenceType.getDescription(main.getLanguageHandle()));
				licenceCodeText.setText(licence.getCode());
				licenceDateText.setText(ui.stringToCurrentDateFormat(licence.getDate(main.getLanguageHandle())));
				licenceDateOfChangeText.setText(ui.stringToCurrentDateFormat(licence.getDateOfChange(main.getLanguageHandle())));
				licenceReasonText.setText(licence.getReason(main.getLanguageHandle()));
				if (licence.getOperational() == 1) {
					operationalCheckBox.setSelected(true);
				} else {
					operationalCheckBox.setSelected(false);
				}

			} catch (Exception e) {
				Main.generalErrorHandler(e);
				return;
			}
		} else {
			licenceTypeText.setText("");
			licenceCodeText.setText("");
			licenceDateText.setText("");
			licenceDateOfChangeText.setText("");
			licenceReasonText.setText("");
			operationalCheckBox.setSelected(false);
		}
		setLocationRelativeTo(getParent());
		show();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {// GEN-BEGIN:initComponents
		jPanel1 = new javax.swing.JPanel();
		licenceTypeLabel = new javax.swing.JLabel();
		licenceTypeText = new javax.swing.JTextField();
		licenceCodeLabel = new javax.swing.JLabel();
		licenceCodeText = new javax.swing.JTextField();
		licenceDateLabel = new javax.swing.JLabel();
		licenceDateOfChangeText = new javax.swing.JTextField();
		licenceReasonText = new javax.swing.JTextField();
		licenceDateOfChangeLabel = new javax.swing.JLabel();
		licenceDateText = new javax.swing.JTextField();
		licenceReasonLabel = new javax.swing.JLabel();
		operationalCheckBox = new javax.swing.JCheckBox();
		selLicenceTypeButton = new javax.swing.JButton();
		jPanel2 = new javax.swing.JPanel();
		jPanel3 = new javax.swing.JPanel();
		helpButton = new javax.swing.JButton();
		jPanel4 = new javax.swing.JPanel();
		okButton = new javax.swing.JButton();
		cancelButton = new javax.swing.JButton();

		setTitle(ui.getString("fina2.bank.licence"));
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				closeDialog(evt);
			}
		});

		jPanel1.setLayout(new java.awt.GridBagLayout());

		jPanel1.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(30, 40, 30, 40)));
		licenceTypeLabel.setText(UIManager.formatedHtmlString(ui.getString("fina2.bank.licenceType")));
		licenceTypeLabel.setFont(ui.getFont());
		jPanel1.add(licenceTypeLabel, UIManager.getGridBagConstraints(0, 0, 10, 10, -1, -1, -1, -1, null));

		licenceTypeText.setEditable(false);
		licenceTypeText.setColumns(15);
		licenceTypeText.setFont(ui.getFont());
		jPanel1.add(licenceTypeText, UIManager.getGridBagConstraints(1, 0, -1, -1, -1, -1, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, null));

		licenceCodeLabel.setText(UIManager.formatedHtmlString(ui.getString("fina2.code")));
		licenceCodeLabel.setFont(ui.getFont());
		jPanel1.add(licenceCodeLabel, UIManager.getGridBagConstraints(0, 1, 10, 10, -1, -1, -1, -1, null));

		licenceCodeText.setColumns(15);
		licenceCodeText.setFont(ui.getFont());
		jPanel1.add(licenceCodeText, UIManager.getGridBagConstraints(1, 1, -1, -1, 2, -1, GridBagConstraints.WEST, -1, null));

		licenceDateLabel.setText(UIManager.formatedHtmlString(ui.getString("fina2.bank.licenceDate")));
		licenceDateLabel.setFont(ui.getFont());
		jPanel1.add(licenceDateLabel, UIManager.getGridBagConstraints(0, 2, 10, 10, -1, -1, -1, -1, null));

		licenceDateOfChangeText.setColumns(15);
		licenceDateOfChangeText.setFont(ui.getFont());
		jPanel1.add(licenceDateOfChangeText, UIManager.getGridBagConstraints(3, 2, -1, -1, -1, -1, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, null));

		licenceReasonText.setColumns(40);
		licenceReasonText.setFont(ui.getFont());
		jPanel1.add(licenceReasonText, UIManager.getGridBagConstraints(1, 3, -1, -1, 3, -1, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, null));

		licenceDateOfChangeLabel.setText(ui.getString("fina2.bank.licenceDateOfChange"));
		licenceDateOfChangeLabel.setFont(ui.getFont());
		jPanel1.add(licenceDateOfChangeLabel, UIManager.getGridBagConstraints(2, 2, 10, 10, -1, -1, GridBagConstraints.EAST, -1, new java.awt.Insets(0, 10, 0, 0)));

		licenceDateText.setColumns(15);
		licenceDateText.setFont(ui.getFont());
		jPanel1.add(licenceDateText, UIManager.getGridBagConstraints(1, 2, -1, -1, -1, -1, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, null));

		licenceReasonLabel.setText(ui.getString("fina2.bank.licenceReason"));
		licenceReasonLabel.setFont(ui.getFont());
		jPanel1.add(licenceReasonLabel, UIManager.getGridBagConstraints(0, 3, 10, 10, -1, -1, -1, -1, null));

		operationalCheckBox.setForeground(new java.awt.Color(102, 102, 153));
		operationalCheckBox.setFont(ui.getFont());
		operationalCheckBox.setText(ui.getString("fina2.bank.operational"));
		jPanel1.add(operationalCheckBox, UIManager.getGridBagConstraints(1, 4, -1, -1, 2, -1, -1, -1, null));

		selLicenceTypeButton.setIcon(ui.getIcon("fina2.find"));
		selLicenceTypeButton.setFont(ui.getFont());
		selLicenceTypeButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				selLicenceTypeButtonActionPerformed(evt);
			}
		});

		jPanel1.add(selLicenceTypeButton, new GridBagConstraints());

		getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

		jPanel2.setLayout(new java.awt.BorderLayout());

		helpButton.setIcon(ui.getIcon("fina2.help"));
		helpButton.setFont(ui.getFont());
		helpButton.setText(ui.getString("fina2.help"));
		helpButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				helpButtonActionPerformed(evt);
			}
		});

		jPanel3.add(helpButton);

		jPanel2.add(jPanel3, java.awt.BorderLayout.WEST);

		okButton.setIcon(ui.getIcon("fina2.ok"));
		okButton.setFont(ui.getFont());
		okButton.setText(ui.getString("fina2.ok"));
		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				okButtonActionPerformed(evt);
			}
		});

		jPanel4.add(okButton);

		cancelButton.setIcon(ui.getIcon("fina2.cancel"));
		cancelButton.setFont(ui.getFont());
		cancelButton.setText(ui.getString("fina2.cancel"));
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelButtonActionPerformed(evt);
			}
		});

		jPanel4.add(cancelButton);

		jPanel2.add(jPanel4, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

		pack();
	}// GEN-END:initComponents

	private void selLicenceTypeButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:
		// event_selLicenceTypeButtonActionPerformed
		licenceTypeDialog.show();
		TableRow row = licenceTypeDialog.getTableRow();
		if (row == null) {
			licenceTypePK = null;
			return;
		}
		licenceTypePK = (LicenceTypePK) row.getPrimaryKey();
		licenceTypeText.setText(row.getValue(0));

	}// GEN-LAST:event_selLicenceTypeButtonActionPerformed

	private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-
		// FIRST
		// :
		// event_okButtonActionPerformed

		if (canAmend) {
			// ===========licence type validation=================
			if (licenceTypeText.getText().length() == 0) {
				ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.bank.LicenceType"));
				return;
			}
			// ===========code validation=================
			if (!ui.isValidCode(licenceCodeText.getText())) {
				if (!ui.isValidLength(licenceCodeText.getText(), false)) {
					ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.invalidLengthCode"));
					return;
				} else if (licenceCodeText.getText().length() == 0) {
					ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.definiton.code"));
					return;
				} else {
					ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.invalidCode"));
					return;
				}
			}
			// ===========date validation=================
			if (licenceDateText.getText().trim().length() == 0) {
				ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.bank.LicenceType.date"));
				return;
			}

			if (!ui.isCorrectDate(licenceDateText.getText().trim())) {
				ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.invalidDateFormat"));
				return;
			}
			// ===========change of date validation=================
			if (!ui.isCorrectDate(licenceDateOfChangeText.getText().trim()) && licenceDateOfChangeText.getText().trim().length() > 0) {
				ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.invalidDateFormat"));
				return;
			}
			// ===========reason validation=================
			if (!ui.isValidLength(licenceReasonText.getText(), true)) {
				ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.bank.LicenceType.reason"));
				return;
			}

			try {
				InitialContext jndi = fina2.Main.getJndiContext();
				Object ref = jndi.lookup("fina2/bank/Licence");
				LicenceHome home = (LicenceHome) PortableRemoteObject.narrow(ref, LicenceHome.class);

				if (node == null) {
					licence = home.create();
				}
				UserTransaction trans = main.getUserTransaction(jndi);
				trans.begin();

				try {
					if (licenceTypePK != null)
						licence.setTypePK(licenceTypePK);
					licence.setCode(licenceCodeText.getText().trim());
					licence.setDate(main.getLanguageHandle(), ui.stringFormatedToServerDataFormat(licenceDateText.getText().trim()));
					licence.setDateOfChange(main.getLanguageHandle(), ui.stringFormatedToServerDataFormat(licenceDateOfChangeText.getText().trim()));
					licence.setReason(main.getLanguageHandle(), licenceReasonText.getText().trim());
					if (operationalCheckBox.isSelected())
						licence.setOperational(1);
					else
						licence.setOperational(0);
					licence.setBankPK(pk);

					if (licenceTypePK == null) {
						trans.rollback();
						if (node == null) {
							licence.remove();
							licence = null;
						}

					}

					trans.commit();
				} catch (Exception e) {
					trans.rollback();
					if (node == null) {
						licence.remove();
						licence = null;
					}
					Main.generalErrorHandler(e);
					return;
				}
				if (node == null) {
					node = new Node(licence.getPrimaryKey(), licenceTypeText.getText(), new Integer(2));
					licenceTypePK = licence.getTypePK();
				} else {
					node.setLabel(licenceTypeText.getText());
					licenceTypePK = licence.getTypePK();
				}
			} catch (Exception e) {
				Main.generalErrorHandler(e);
			}
		}

		dispose();

	}// GEN-LAST:event_okButtonActionPerformed

	private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN
		// -
		// FIRST
		// :
		// event_cancelButtonActionPerformed
		ok = false;
		setVisible(false);
		dispose();
	}// GEN-LAST:event_cancelButtonActionPerformed

	private void helpButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN
		// -
		// FIRST
		// :
		// event_helpButtonActionPerformed
		// Add your handling code here:
	}// GEN-LAST:event_helpButtonActionPerformed

	/** Closes the dialog */
	private void closeDialog(java.awt.event.WindowEvent evt) {// GEN-FIRST:
		// event_closeDialog
		setVisible(false);
		dispose();
	}// GEN-LAST:event_closeDialog

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JPanel jPanel1;
	private javax.swing.JLabel licenceTypeLabel;
	private javax.swing.JTextField licenceTypeText;
	private javax.swing.JLabel licenceCodeLabel;
	private javax.swing.JTextField licenceCodeText;
	private javax.swing.JLabel licenceDateLabel;
	private javax.swing.JTextField licenceDateOfChangeText;
	private javax.swing.JTextField licenceReasonText;
	private javax.swing.JLabel licenceDateOfChangeLabel;
	private javax.swing.JTextField licenceDateText;
	private javax.swing.JLabel licenceReasonLabel;
	private javax.swing.JCheckBox operationalCheckBox;
	private javax.swing.JButton selLicenceTypeButton;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JButton helpButton;
	private javax.swing.JPanel jPanel4;
	private javax.swing.JButton okButton;
	private javax.swing.JButton cancelButton;
	// End of variables declaration//GEN-END:variables
}
