/*
 * LanguageAmendDialog.java
 *
 * Created on October 19, 2001, 6:57 PM
 */

package fina2.i18n;

import java.awt.GridBagConstraints;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.transaction.UserTransaction;

import fina2.Main;
import fina2.ui.UIManager;
import fina2.ui.table.TableRow;
import fina2.ui.table.TableRowImpl;

/**
 * 
 * @author David Shalamberidze
 */
public class LanguageAmendDialog extends javax.swing.JDialog {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;
	private fina2.Main main = fina2.Main.main;
	private static String defaultLanguage;
	private TableRow tableRow;
	private Language lang;

	/** Creates new form LanguageAmendDialog */
	public LanguageAmendDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);
		ui.loadIcon("fina2.ok", "ok.gif");
		ui.loadIcon("fina2.cancel", "cancel.gif");
		ui.loadIcon("fina2.help", "help.gif");
		initComponents();
		String[] dates = { "dd/MM/yyyy", "dd.MM.yyyy", "MM/dd/yyyy", "MMM dd, yyyy", "yyyy.MMM.dd", "dd/MM/yy", "dd.MM.yy", "MM/dd/yy", "MMM dd, yy", "yy.MMM.dd" };
		dateFormatList.setModel(new javax.swing.DefaultComboBoxModel(dates));
		String[] numbers = { "#0.0", "#0.00", "#0.000", "#0.0000", "#.0", "#.00", "#.000", "#.0000", "###,##0.0", "###,##0.00", "###,##0.000", "###,##0.0000" };
		numberFormatList.setModel(new javax.swing.DefaultComboBoxModel(numbers));

		String[] encodings = { "ASCII", "Cp1252", "ISO8859_1",
		/*
		 * "UnicodeBig", "UnicodeBigUnmarked", "UnicodeLittle",
		 * "UnicodeLittleUnmarked",
		 */
		"UTF8", "UTF-16", "Big5", "Cp037", "Cp273", "Cp277", "Cp278", "Cp280", "Cp284", "Cp285", "Cp297", "Cp420", "Cp424", "Cp437", "Cp500", "Cp737", "Cp775", "Cp838", "Cp850", "Cp852", "Cp855",
				"Cp856", "Cp857", "Cp858", "Cp860", "Cp861", "Cp862", "Cp863", "Cp864", "Cp865", "Cp866", "Cp868", "Cp869", "Cp870", "Cp871", "Cp874", "Cp875", "Cp918", "Cp921", "Cp922", "Cp930",
				"Cp933", "Cp935", "Cp937", "Cp939", "Cp942", "Cp942C", "Cp943", "Cp943C", "Cp948", "Cp949", "Cp949C", "Cp950", "Cp964", "Cp970", "Cp1006", "Cp1025", "Cp1026", "Cp1046", "Cp1097",
				"Cp1098", "Cp1112", "Cp1122", "Cp1123", "Cp1124", "Cp1140", "Cp1141", "Cp1142", "Cp1143", "Cp1144", "Cp1145", "Cp1146", "Cp1147", "Cp1148", "Cp1149", "Cp1250", "Cp1251", "Cp1253",
				"Cp1254", "Cp1255", "Cp1256", "Cp1257", "Cp1258", "Cp1381", "Cp1383", "Cp33722", "EUC_CN", "EUC_JP", "EUC_KR", "EUC_TW", "GBK", "ISO2022CN", "ISO2022CN_CNS", "ISO2022CN_GB",
				"ISO2022JP", "ISO2022KR", "ISO8859_2", "ISO8859_3", "ISO8859_4", "ISO8859_5", "ISO8859_6", "ISO8859_7", "ISO8859_8", "ISO8859_9", "ISO8859_13", "ISO8859_15_FDIS", "JIS0201",
				"JIS0208", "JIS0212", "Johab", "KOI8_R", "MS874", "MS932", "MS936", "MS949", "MS950", "MacArabic", "MacCentralEurope", "MacCroatian", "MacCyrillic", "MacDingbat", "MacGreek",
				"MacHebrew", "MacIceland", "MacRoman", "MacRomania", "MacSymbol", "MacThai", "MacTurkish", "MacUkraine", "SJIS", "TIS620" };
		htmlCharsetList.setModel(new javax.swing.DefaultComboBoxModel(encodings));
		xmlEncodingList.setModel(new javax.swing.DefaultComboBoxModel(encodings));

	}

	public TableRow getTableRow() {
		return tableRow;
	}

	public void show(TableRow tableRow, boolean canAmend) {
		this.tableRow = tableRow;
		codeText.setEditable(canAmend);
		descriptionText.setEditable(canAmend);
		fontFaceText.setEditable(canAmend);
		fontSizeText.setEditable(canAmend);
		dateFormatList.setEnabled(canAmend);
		numberFormatList.setEnabled(canAmend);
		htmlCharsetList.setEnabled(canAmend);
		xmlEncodingList.setEnabled(canAmend);
		lang = null;
		if (tableRow != null) {

			LanguagePK pk = (LanguagePK) tableRow.getPrimaryKey();

			try {
				InitialContext jndi = fina2.Main.getJndiContext();

				Object ref = jndi.lookup("fina2/i18n/Language");
				LanguageHome home = (LanguageHome) PortableRemoteObject.narrow(ref, LanguageHome.class);

				lang = home.findByPrimaryKey(pk);

				codeText.setText(lang.getCode());
				defaultLanguage = lang.getCode();
				descriptionText.setText(lang.getDescription());
				fontFaceText.setText(lang.getFontFace());
				fontSizeText.setText(String.valueOf(lang.getFontSize()));
				dateFormatList.setSelectedItem(lang.getDateFormat());
				numberFormatList.setSelectedItem(lang.getNumberFormat());
				htmlCharsetList.setSelectedItem(lang.getHtmlCharset());
				xmlEncodingList.setSelectedItem(lang.getXmlEncoding());
			} catch (Exception e) {
				Main.generalErrorHandler(e);
				return;
			}
		}
		setLocationRelativeTo(getParent());
		show();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {// GEN-BEGIN:initComponents
		jPanel1 = new javax.swing.JPanel();
		jPanel3 = new javax.swing.JPanel();
		helpButton = new javax.swing.JButton();
		jPanel4 = new javax.swing.JPanel();
		okButton = new javax.swing.JButton();
		cancelButton = new javax.swing.JButton();
		jPanel2 = new javax.swing.JPanel();
		jLabel1 = new javax.swing.JLabel();
		codeText = new javax.swing.JTextField();
		jLabel2 = new javax.swing.JLabel();
		descriptionText = new javax.swing.JTextField();
		jLabel3 = new javax.swing.JLabel();
		fontFaceText = new javax.swing.JTextField();
		jLabel4 = new javax.swing.JLabel();
		fontSizeText = new javax.swing.JTextField();
		jLabel5 = new javax.swing.JLabel();
		jLabel6 = new javax.swing.JLabel();
		jLabel7 = new javax.swing.JLabel();
		jLabel8 = new javax.swing.JLabel();
		dateFormatList = new javax.swing.JComboBox();
		numberFormatList = new javax.swing.JComboBox();
		htmlCharsetList = new javax.swing.JComboBox();
		xmlEncodingList = new javax.swing.JComboBox();
		jButton1 = new javax.swing.JButton();
		jLabel9 = new javax.swing.JLabel();

		setTitle(ui.getString("fina2.i18n.language"));
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				closeDialog(evt);
			}
		});

		jPanel1.setLayout(new java.awt.BorderLayout());

		helpButton.setIcon(ui.getIcon("fina2.help"));
		helpButton.setFont(ui.getFont());
		helpButton.setText(ui.getString("fina2.help"));
		helpButton.setEnabled(false);
		jPanel3.add(helpButton);

		jPanel1.add(jPanel3, java.awt.BorderLayout.WEST);

		okButton.setIcon(ui.getIcon("fina2.ok"));
		okButton.setFont(ui.getFont());
		okButton.setText(ui.getString("fina2.ok"));
		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				okButtonActionPerformed(evt);
			}
		});

		jPanel4.add(okButton);

		cancelButton.setIcon(ui.getIcon("fina2.cancel"));
		cancelButton.setFont(ui.getFont());
		cancelButton.setText(ui.getString("fina2.cancel"));
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelButtonActionPerformed(evt);
			}
		});

		jPanel4.add(cancelButton);

		jPanel1.add(jPanel4, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

		jPanel2.setLayout(new java.awt.GridBagLayout());

		jPanel2.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(30, 70, 30, 70)));
		jLabel1.setFont(ui.getFont());
		jLabel1.setText(UIManager.formatedHtmlString(ui.getString("fina2.code")));
		jPanel2.add(jLabel1, UIManager.getGridBagConstraints(0, 0, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		codeText.setColumns(8);
		codeText.setFont(ui.getFont());
		jPanel2.add(codeText, UIManager.getGridBagConstraints(1, 0, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		jLabel2.setText(ui.getString("fina2.description"));
		jLabel2.setFont(ui.getFont());
		jPanel2.add(jLabel2, UIManager.getGridBagConstraints(0, 1, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		descriptionText.setColumns(15);
		descriptionText.setFont(ui.getFont());
		jPanel2.add(descriptionText, UIManager.getGridBagConstraints(1, 1, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		jLabel3.setFont(ui.getFont());
		jLabel3.setText(ui.getString("fina2.i18n.fontFace"));
		jPanel2.add(jLabel3, UIManager.getGridBagConstraints(0, 2, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		fontFaceText.setColumns(15);
		fontFaceText.setFont(ui.getFont());
		jPanel2.add(fontFaceText, UIManager.getGridBagConstraints(1, 2, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		jLabel4.setFont(ui.getFont());
		jLabel4.setText(UIManager.formatedHtmlString(ui.getString("fina2.i18n.fontSize")));
		jPanel2.add(jLabel4, UIManager.getGridBagConstraints(0, 3, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		fontSizeText.setColumns(4);
		fontSizeText.setFont(ui.getFont());
		jPanel2.add(fontSizeText, UIManager.getGridBagConstraints(1, 3, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		jLabel5.setFont(ui.getFont());
		jLabel5.setText(ui.getString("fina2.i18n.dateFormat"));
		jPanel2.add(jLabel5, UIManager.getGridBagConstraints(0, 4, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		jLabel6.setFont(ui.getFont());
		jLabel6.setText(ui.getString("fina2.i18n.numberFormat"));
		jPanel2.add(jLabel6, UIManager.getGridBagConstraints(0, 5, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		jLabel7.setFont(ui.getFont());
		jLabel7.setText(ui.getString("fina2.i18n.htmlCharset"));
		jPanel2.add(jLabel7, UIManager.getGridBagConstraints(0, 6, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		jLabel8.setFont(ui.getFont());
		jLabel8.setText(ui.getString("fina2.i18n.xmlEncoding"));
		jPanel2.add(jLabel8, UIManager.getGridBagConstraints(0, 7, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		dateFormatList.setEditable(true);
		dateFormatList.setFont(ui.getFont());
		jPanel2.add(dateFormatList, UIManager.getGridBagConstraints(1, 4, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		numberFormatList.setEditable(true);
		numberFormatList.setFont(ui.getFont());
		jPanel2.add(numberFormatList, UIManager.getGridBagConstraints(1, 5, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		htmlCharsetList.setEditable(true);
		htmlCharsetList.setFont(ui.getFont());
		jPanel2.add(htmlCharsetList, UIManager.getGridBagConstraints(1, 6, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		xmlEncodingList.setEditable(true);
		xmlEncodingList.setFont(ui.getFont());
		jPanel2.add(xmlEncodingList, UIManager.getGridBagConstraints(1, 7, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		jButton1.setFont(ui.getFont());
		jButton1.setText(ui.getString("fina2.amend"));
		jButton1.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				jButton1ActionPerformed(evt);
			}
		});

		jPanel2.add(jButton1, UIManager.getGridBagConstraints(1, 8, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		jLabel9.setFont(ui.getFont());
		jLabel9.setText("Bundle");
		jPanel2.add(jLabel9, UIManager.getGridBagConstraints(0, 8, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);

		pack();
	}// GEN-END:initComponents

	private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {// GEN-
		// FIRST
		// :
		// event_jButton1ActionPerformed
		LanguageBundleAmendDialog bDialog = new LanguageBundleAmendDialog(null, true);
		bDialog.show(lang);
	}// GEN-LAST:event_jButton1ActionPerformed

	private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-
		// FIRST
		// :
		// event_okButtonActionPerformed
		if (!fina2.Main.isValidCode(codeText.getText())) {
			ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.invalidCode"));
			return;
		}
		try {
			InitialContext jndi = fina2.Main.getJndiContext();
			Object ref = jndi.lookup("fina2/i18n/Language");
			LanguageHome home = (LanguageHome) PortableRemoteObject.narrow(ref, LanguageHome.class);

			try {
				if (lang == null) {
					lang = home.create();
				}
			} catch (Exception e) {
				if (tableRow == null)
					lang = null;
				Main.generalErrorHandler(e);
				return;
			}
			UserTransaction trans = main.getUserTransaction(jndi);
			trans.begin();

			try {
				lang.setCode(codeText.getText());
				lang.setDescription(descriptionText.getText());
				lang.setFontFace(fontFaceText.getText());
				try {
					lang.setFontSize(Integer.valueOf(fontSizeText.getText()).intValue());
				} catch (NumberFormatException e) {
					try {
						trans.rollback();
					} catch (Exception tex) {
						tex.printStackTrace();
					}
					if (tableRow == null) {
						lang.remove();
						lang = null;
					}
					Main.errorHandler(null, Main.getString("fina2.title"), Main.getString("fina2.i18n.fontSizeFormatException"));
					return;
				} catch (Exception e) {
					try {
						trans.rollback();
					} catch (Exception tex) {
					}
					if (tableRow == null) {
						lang.remove();
						lang = null;
					}
					Main.generalErrorHandler(e);
					return;
				}
				lang.setDateFormat((String) dateFormatList.getSelectedItem());
				lang.setNumberFormat((String) numberFormatList.getSelectedItem());
				lang.setHtmlCharset((String) htmlCharsetList.getSelectedItem());
				lang.setXmlEncoding((String) xmlEncodingList.getSelectedItem());
				trans.commit();
			} catch (Exception e) {
				try {
					trans.rollback();
				} catch (Exception tex) {
					tex.printStackTrace();
				}
				if (tableRow == null) {
					lang.remove();
					lang = null;
				}
				Main.generalErrorHandler(e);
				return;
			}
			if (tableRow == null) {
				tableRow = new TableRowImpl(lang.getPrimaryKey(), 2);
				tableRow.setValue(0, lang.getCode());
				tableRow.setValue(1, lang.getDescription());
				if (ui.getDefaultLanguage() != null && defaultLanguage.equals(ui.getDefaultLanguage()))
					ui.setDefaultLanguage(lang.getCode());
			} else {
				tableRow.setValue(0, lang.getCode());
				tableRow.setValue(1, lang.getDescription());
				if (ui.getDefaultLanguage() != null && defaultLanguage.equals(ui.getDefaultLanguage()))
					ui.setDefaultLanguage(lang.getCode());
			}
		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
		dispose();
	}// GEN-LAST:event_okButtonActionPerformed

	private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN
		// -
		// FIRST
		// :
		// event_cancelButtonActionPerformed
		dispose();
	}// GEN-LAST:event_cancelButtonActionPerformed

	/** Closes the dialog */
	private void closeDialog(java.awt.event.WindowEvent evt) {// GEN-FIRST:
		// event_closeDialog
		setVisible(false);
		dispose();
	}// GEN-LAST:event_closeDialog

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton cancelButton;
	private javax.swing.JTextField codeText;
	private javax.swing.JComboBox dateFormatList;
	private javax.swing.JTextField descriptionText;
	private javax.swing.JTextField fontFaceText;
	private javax.swing.JTextField fontSizeText;
	private javax.swing.JButton helpButton;
	private javax.swing.JComboBox htmlCharsetList;
	private javax.swing.JButton jButton1;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JLabel jLabel2;
	private javax.swing.JLabel jLabel3;
	private javax.swing.JLabel jLabel4;
	private javax.swing.JLabel jLabel5;
	private javax.swing.JLabel jLabel6;
	private javax.swing.JLabel jLabel7;
	private javax.swing.JLabel jLabel8;
	private javax.swing.JLabel jLabel9;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JPanel jPanel4;
	private javax.swing.JComboBox numberFormatList;
	private javax.swing.JButton okButton;
	private javax.swing.JComboBox xmlEncodingList;
	// End of variables declaration//GEN-END:variables

}
