/*
 * ComparisonAmendFrame.java
 *
 * Created on December 25, 2001, 11:41 AM
 */

package fina2.metadata;

import java.awt.GridBagConstraints;
import java.util.Collection;
import java.util.Vector;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.swing.ListSelectionModel;

import fina2.BaseFrame;
import fina2.Main;
import fina2.ui.UIManager;
import fina2.ui.table.EJBTable;
import fina2.ui.table.TableReviewFrame;
import fina2.ui.tree.Node;

/**
 * 
 * @author Shota Shalamberidze
 */
public class ComparisonsAmendFrame extends BaseFrame {
	private fina2.ui.UIManager ui = fina2.Main.main.ui;
	private fina2.Main main = fina2.Main.main;

	private EJBTable table;
	private MDTNode mdtNode;
	private Node node;

	private boolean canAmend = false;
	private boolean canDelete = false;
	private boolean canReview = false;

	private MDTNodePK pk;
	private NodeComparisonAmendDialog comparisonDialog;

	/** Creates new form ComparisonAmendFrame */
	public ComparisonsAmendFrame() {
		ui.loadIcon("fina2.help", "help.gif");
		ui.loadIcon("fina2.print", "print.gif");
		ui.loadIcon("fina2.refresh", "refresh.gif");
		ui.loadIcon("fina2.close", "cancel.gif");

		ui.loadIcon("fina2.amend", "amend.gif");
		ui.loadIcon("fina2.create", "insert.gif");
		ui.loadIcon("fina2.review", "review.gif");
		ui.loadIcon("fina2.delete", "delete.gif");

		table = new EJBTable();

		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		table
				.addSelectionListener(new javax.swing.event.ListSelectionListener() {
					public void valueChanged(
							javax.swing.event.ListSelectionEvent evt) {
						if (table.getSelectedRow() == -1) {
							reviewButton.setEnabled(false);
							amendButton.setEnabled(false);
							deleteButton.setEnabled(false);
						} else {
							amendButton.setEnabled(canAmend);
							deleteButton.setEnabled(canDelete);
							reviewButton.setEnabled(canReview || canAmend);
						}
					}
				});

		initComponents();
		// createButton.setEnabled(false);
		scrollPane.setViewportView(table);
		BaseFrame.ensureVisible(this);

	}

	private void initTable() {
		try {
			InitialContext jndi = fina2.Main.getJndiContext();
			Object ref = jndi.lookup("fina2/metadata/MDTSession");
			MDTSessionHome home = (MDTSessionHome) PortableRemoteObject.narrow(
					ref, MDTSessionHome.class);

			MDTSession session = home.create();

			Vector colNames = new Vector();
			colNames.add(ui.getString("fina2.code"));
			colNames.add(ui.getString("fina2.condition"));
			colNames.add(ui.getString("fina2.equation"));

			Collection rows = new Vector();

			table.initTable(colNames, session.getComparisons(main
					.getUserHandle(), new MDTNodePK(-1)));

		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
	}

	public void show() {
		if (isVisible())
			return;
		try {
			fina2.security.User user = (fina2.security.User) main
					.getUserHandle().getEJBObject();
			canAmend = user.hasPermission("fina2.metadata.amend");
			canDelete = user.hasPermission("fina2.metadata.delete");
			canReview = user.hasPermission("fina2.metadata.review");
		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
		amendButton.setEnabled(canAmend);
		amendButton.setVisible(canAmend);

		deleteButton.setEnabled(canDelete);
		deleteButton.setVisible(canDelete);

		reviewButton.setEnabled(canAmend || canReview);
		printButton.setEnabled(canAmend || canReview);
		reviewButton.setVisible(canAmend || canReview);
		printButton.setVisible(canAmend || canReview);

		initTable();

		super.show();

	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {// GEN-BEGIN:initComponents
		jPanel1 = new javax.swing.JPanel();
		jPanel2 = new javax.swing.JPanel();
		jPanel3 = new javax.swing.JPanel();
		jPanel4 = new javax.swing.JPanel();
		amendButton = new javax.swing.JButton();
		reviewButton = new javax.swing.JButton();
		deleteButton = new javax.swing.JButton();
		jPanel5 = new javax.swing.JPanel();
		scrollPane = new javax.swing.JScrollPane();
		jPanel8 = new javax.swing.JPanel();
		jPanel9 = new javax.swing.JPanel();
		jPanel10 = new javax.swing.JPanel();

		setTitle(ui.getString("fina2.metadata.comparisonrules"));
		setFont(ui.getFont());
		initBaseComponents();
		getContentPane().add(jPanel1, java.awt.BorderLayout.NORTH);

		jPanel2.setLayout(new java.awt.BorderLayout());

		jPanel2.setFont(ui.getFont());
		jPanel3.setLayout(new java.awt.BorderLayout());

		jPanel4.setLayout(new java.awt.GridBagLayout());

		amendButton.setIcon(ui.getIcon("fina2.amend"));
		amendButton.setFont(ui.getFont());
		amendButton.setText(ui.getString("fina2.amend"));
		amendButton.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
		amendButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				amendButtonActionPerformed(evt);
			}
		});

		jPanel4.add(amendButton, UIManager.getGridBagConstraints(0, 0, -1, -1,
				-1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(
						0, 5, 0, 5)));

		reviewButton.setIcon(ui.getIcon("fina2.review"));
		reviewButton.setFont(ui.getFont());
		reviewButton.setText(ui.getString("fina2.review"));
		reviewButton.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
		reviewButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				reviewButtonActionPerformed(evt);
			}
		});

		jPanel4.add(reviewButton, UIManager.getGridBagConstraints(0, 1, -1, -1,
				-1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(
						5, 5, 0, 5)));

		deleteButton.setIcon(ui.getIcon("fina2.delete"));
		deleteButton.setFont(ui.getFont());
		deleteButton.setText(ui.getString("fina2.delete"));
		deleteButton.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
		deleteButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				deleteButtonActionPerformed(evt);
			}
		});
		jPanel4.add(deleteButton, UIManager.getGridBagConstraints(0, 2, -1, -1,
				-1, -1, -1, GridBagConstraints.HORIZONTAL, new java.awt.Insets(
						5, 5, 0, 5)));

		jPanel3.add(jPanel4, java.awt.BorderLayout.NORTH);

		jPanel2.add(jPanel3, java.awt.BorderLayout.EAST);

		jPanel5.setLayout(new java.awt.BorderLayout());

		jPanel5.add(scrollPane, java.awt.BorderLayout.CENTER);

		jPanel2.add(jPanel5, java.awt.BorderLayout.CENTER);

		getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);

		jPanel8.setLayout(new java.awt.BorderLayout());

		if (ui.getHelpManager().IsHelpSystem()) {
			ui.getHelpManager().createDisplayHelpFromFocus(helpButton, "Comparison_Rules");
		} else {
			helpButton.setEnabled(false);
		}
		jPanel9.add(helpButton);

		jPanel8.add(jPanel9, java.awt.BorderLayout.WEST);

		jPanel10.add(printButton);

		jPanel10.add(refreshButton);

		jPanel10.add(closeButton);

		jPanel8.add(jPanel10, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel8, java.awt.BorderLayout.SOUTH);

	}// GEN-END:initComponents

	protected void printButtonActionPerformed(java.awt.event.ActionEvent evt) {
		TableReviewFrame printFrame = new TableReviewFrame();
		printFrame.show(getTitle(), table);
	}

	protected void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {
		formComponentHidden(null);
		dispose();
	}

	protected void refreshButtonActionPerformed(java.awt.event.ActionEvent evt) {
		initTable();
	}

	protected void helpButtonActionPerformed(java.awt.event.ActionEvent evt) {

	}

	private void deleteButtonActionPerformed(java.awt.event.ActionEvent evt) {
		if (!ui.showConfirmBox(main.getMainFrame(), ui
				.getString("fina2.metadata.comparisonDeleteQuestion")))
			return;
		try {
			InitialContext jndi = fina2.Main.getJndiContext();
			Object ref = jndi.lookup("fina2/metadata/MDTSession");
			MDTSessionHome sessionHome = (MDTSessionHome) PortableRemoteObject
					.narrow(ref, MDTSessionHome.class);
			MDTSession session = sessionHome.create();
			session.removeComparison(table.getSelectedTableRow());
			table.removeRow(table.getSelectedRow());
		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
	}

	private void reviewButtonActionPerformed(java.awt.event.ActionEvent evt) {

		comparisonDialog = new NodeComparisonAmendDialog(table, table
				.getSelectedTableRow(), table.getSelectedRow(), main
				.getMainFrame(), true);

		comparisonDialog.show(table.getSelectedTableRow().getValue(0), table
				.getSelectedTableRow().getValue(1), table.getSelectedTableRow()
				.getValue(2), false);

	}

	private void amendButtonActionPerformed(java.awt.event.ActionEvent evt) {

		comparisonDialog = new NodeComparisonAmendDialog(table, table
				.getSelectedTableRow(), table.getSelectedRow(), main
				.getMainFrame(), true);

		comparisonDialog.show(table.getSelectedTableRow().getValue(0), table
				.getSelectedTableRow().getValue(1), table.getSelectedTableRow()
				.getValue(2), true);

	}

	/** Exit the Application */

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JPanel jPanel4;
	private javax.swing.JButton amendButton;
	private javax.swing.JButton reviewButton;
	private javax.swing.JButton deleteButton;
	private javax.swing.JPanel jPanel5;
	private javax.swing.JScrollPane scrollPane;
	private javax.swing.JPanel jPanel8;
	private javax.swing.JPanel jPanel9;
	private javax.swing.JPanel jPanel10;
	// End of variables declaration//GEN-END:variables

}
