/*
 * MenuAmendDialog.java
 *
 * Created on November 1, 2001, 3:59 PM
 */

package fina2.ui.menu;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.transaction.UserTransaction;

import fina2.Main;
import fina2.ui.UIManager;
import fina2.ui.tree.Node;

/**
 * 
 * @author David Shalamberidze
 */
public class MenuAmendDialog extends javax.swing.JDialog {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;
	private fina2.Main main = fina2.Main.main;

	private Menu menu;
	private Node parent;
	private Node node;

	public MenuAmendDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);

		ui.loadIcon("fina2.ok", "ok.gif");
		ui.loadIcon("fina2.cancel", "cancel.gif");
		ui.loadIcon("fina2.help", "help.gif");
		ui.loadIcon("fina2.lookup", "find.gif");
		initComponents();
	}

	public Node getNode() {
		return node;
	}

	public void show(Node parent, Node node) {
		this.node = node;
		this.parent = parent;

		if (node != null) {

			try {
				InitialContext jndi = fina2.Main.getJndiContext();

				Object ref = jndi.lookup("fina2/ui/menu/Menu");
				MenuHome home = (MenuHome) PortableRemoteObject.narrow(ref, MenuHome.class);

				MenuPK pk = (MenuPK) node.getPrimaryKey();
				menu = home.findByPrimaryKey(pk);

				descriptionText.setText(menu.getDescription(main.getLanguageHandle()));

			} catch (Exception e) {
				Main.generalErrorHandler(e);
				return;
			}
		}
		setLocationRelativeTo(getParent());
		show();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {// GEN-BEGIN:initComponents

		jPanel1 = new javax.swing.JPanel();
		jPanel3 = new javax.swing.JPanel();
		helpButton = new javax.swing.JButton();
		jPanel4 = new javax.swing.JPanel();
		okButton = new javax.swing.JButton();
		cancelButton = new javax.swing.JButton();
		jPanel2 = new javax.swing.JPanel();
		jLabel1 = new javax.swing.JLabel();
		descriptionText = new javax.swing.JTextField();

		setTitle(ui.getString("fina2.ui.menu.menu"));
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				closeDialog(evt);
			}
		});

		jPanel1.setLayout(new java.awt.BorderLayout());

		helpButton.setIcon(ui.getIcon("fina2.help"));
		helpButton.setFont(ui.getFont());
		helpButton.setText(ui.getString("fina2.help"));
		helpButton.setEnabled(false);
		jPanel3.add(helpButton);

		jPanel1.add(jPanel3, java.awt.BorderLayout.WEST);

		okButton.setIcon(ui.getIcon("fina2.ok"));
		okButton.setFont(ui.getFont());
		okButton.setText(ui.getString("fina2.ok"));
		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				okButtonActionPerformed(evt);
			}
		});

		jPanel4.add(okButton);

		cancelButton.setIcon(ui.getIcon("fina2.cancel"));
		cancelButton.setFont(ui.getFont());
		cancelButton.setText(ui.getString("fina2.cancel"));
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelButtonActionPerformed(evt);
			}
		});

		jPanel4.add(cancelButton);

		jPanel1.add(jPanel4, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

		jPanel2.setLayout(new java.awt.GridBagLayout());

		jPanel2.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(30, 70, 30, 70)));
		jLabel1.setText(UIManager.formatedHtmlString(ui.getString("fina2.description")));
		jLabel1.setFont(ui.getFont());
		jPanel2.add(jLabel1, UIManager.getGridBagConstraints(0, 0, 10, 10, -1, -1, -1, -1, null));

		descriptionText.setColumns(15);
		descriptionText.setFont(ui.getFont());
		jPanel2.add(descriptionText, UIManager.getGridBagConstraints(1, 0, -1, -1, -1, -1, -1, -1, null));

		getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);

		pack();
	}// GEN-END:initComponents

	private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN
																				// -
																				// FIRST
																				// :
																				// event_cancelButtonActionPerformed
		dispose();
	}// GEN-LAST:event_cancelButtonActionPerformed

	private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {
		if (!ui.isValidLength(descriptionText.getText(), true)) {
			ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.web.banktype.description.length"));
			return;
		} else if (descriptionText.getText().trim().length() == 0) {
			ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.pleaseEnterDescription"));
			return;
		}

		// event_okButtonActionPerformed
		try {
			InitialContext jndi = fina2.Main.getJndiContext();

			Object ref = jndi.lookup("fina2/ui/menu/Menu");
			MenuHome home = (MenuHome) PortableRemoteObject.narrow(ref, MenuHome.class);

			try {
				if (node == null) {
					MenuPK parentPK = (MenuPK) parent.getPrimaryKey();
					menu = home.create(parentPK);
					// menu.setType(MenuConstants.MENU_ACTION_TYPE);
				}
			} catch (Exception e) {
				if (node == null)
					menu = null;
				Main.generalErrorHandler(e);
				return;
			}
			UserTransaction trans = main.getUserTransaction(jndi);
			trans.begin();
			try {
				menu.setDescription(main.getLanguageHandle(), descriptionText.getText());
				trans.commit();
			} catch (Exception e) {
				Main.generalErrorHandler(e);
				trans.rollback();
				if (node == null) {
					menu.remove();
					menu = null;
				}
				return;
			}
			if (node == null) {
				node = new Node(menu.getPrimaryKey(), descriptionText.getText(), new Integer(menu.getType()));
			} else {
				node.setLabel(descriptionText.getText());
			}
		} catch (Exception e) {
			Main.generalErrorHandler(e);
		}
		dispose();
	}// GEN-LAST:event_okButtonActionPerformed

	private void closeDialog(java.awt.event.WindowEvent evt) {// GEN-FIRST:
																// event_closeDialog
		setVisible(false);
		dispose();
	}// GEN-LAST:event_closeDialog

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JButton cancelButton;
	private javax.swing.JTextField descriptionText;
	private javax.swing.JButton helpButton;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JPanel jPanel4;
	private javax.swing.JButton okButton;
	// End of variables declaration//GEN-END:variables

}
