/*
 * MenuItemAmendDialog.java
 *
 * Created on October 17, 2001, 4:07 PM
 */

package fina2.ui.menu;

import java.awt.GridBagConstraints;
import java.util.Vector;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.swing.AbstractAction;
import javax.transaction.UserTransaction;

import fina2.Main;
import fina2.ui.UIManager;
import fina2.ui.table.TableRow;
import fina2.ui.tree.Node;

/**
 * 
 * @author David Shalamberidze
 */
public class MenuItemAmendDialog extends javax.swing.JDialog {

	private fina2.ui.UIManager ui = fina2.Main.main.ui;
	private fina2.Main main = fina2.Main.main;

	private Menu menu;
	private Node parent;
	private Node node;

	private String actionKey = "";

	/** Creates new form MenuItemAmendDialog */
	public MenuItemAmendDialog(java.awt.Frame parent, boolean modal) {
		super(parent, modal);
		ui.loadIcon("fina2.ok", "ok.gif");
		ui.loadIcon("fina2.cancel", "cancel.gif");
		ui.loadIcon("fina2.help", "help.gif");
		ui.loadIcon("fina2.lookup", "find.gif");
		initComponents();

		Vector types = new Vector(2);
		types.add(ui.getString("fina2.ui.menu.action"));
		types.add(ui.getString("fina2.ui.menu.application"));
		typeList.setModel(new javax.swing.DefaultComboBoxModel(new Vector(types)));

	}

	public Node getNode() {
		return node;
	}

	public void show(Node parent, Node node) {
		this.node = node;
		this.parent = parent;

		if (node != null) {

			try {
				InitialContext jndi = fina2.Main.getJndiContext();

				Object ref = jndi.lookup("fina2/ui/menu/Menu");
				MenuHome home = (MenuHome) PortableRemoteObject.narrow(ref, MenuHome.class);

				MenuPK pk = (MenuPK) node.getPrimaryKey();
				menu = home.findByPrimaryKey(pk);

				descriptionText.setText(menu.getDescription(main.getLanguageHandle()));
				switch (menu.getType()) {
				case MenuConstants.MENU_ACTION_TYPE:

					typeList.setSelectedIndex(0);
					applicationText.setText("");
					applicationText.setEditable(false);
					actionKey = menu.getActionKey();

					AbstractAction action = ui.getAllAction(actionKey);
					if (action != null) {
						actionText.setText((String) action.getValue(javax.swing.AbstractAction.NAME));
					} else {
						actionText.setText("#Unknown");
					}

					break;
				case MenuConstants.MENU_APPLICATION_TYPE:

					typeList.setSelectedIndex(1);
					actionText.setText("");
					applicationText.setEditable(true);
					applicationText.setText(menu.getApplication());
					lookupActionButton.setEnabled(false);

					break;
				default:

					typeList.setSelectedIndex(0);
					applicationText.setEditable(false);
				}

			} catch (Exception e) {
				Main.generalErrorHandler(e);
				return;
			}
		}
		setLocationRelativeTo(getParent());
		show();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {// GEN-BEGIN:initComponents
		jPanel1 = new javax.swing.JPanel();
		jPanel2 = new javax.swing.JPanel();
		helpButton = new javax.swing.JButton();
		jPanel3 = new javax.swing.JPanel();
		okButton = new javax.swing.JButton();
		cancelButton = new javax.swing.JButton();
		jPanel4 = new javax.swing.JPanel();
		jLabel1 = new javax.swing.JLabel();
		descriptionText = new javax.swing.JTextField();
		jLabel2 = new javax.swing.JLabel();
		typeList = new javax.swing.JComboBox();
		jLabel3 = new javax.swing.JLabel();
		jLabel4 = new javax.swing.JLabel();
		applicationText = new javax.swing.JTextField();
		actionText = new javax.swing.JTextField();
		lookupActionButton = new javax.swing.JButton();

		setTitle(ui.getString("fina2.ui.menu.menuItem"));
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				closeDialog(evt);
			}
		});

		jPanel1.setLayout(new java.awt.BorderLayout());

		helpButton.setIcon(ui.getIcon("fina2.help"));
		helpButton.setFont(ui.getFont());
		helpButton.setText(ui.getString("fina2.help"));
		helpButton.setEnabled(false);
		jPanel2.add(helpButton);

		jPanel1.add(jPanel2, java.awt.BorderLayout.WEST);

		okButton.setIcon(ui.getIcon("fina2.ok"));
		okButton.setFont(ui.getFont());
		okButton.setText(ui.getString("fina2.ok"));
		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				okButtonActionPerformed(evt);
			}
		});

		jPanel3.add(okButton);

		cancelButton.setIcon(ui.getIcon("fina2.cancel"));
		cancelButton.setFont(ui.getFont());
		cancelButton.setText(ui.getString("fina2.cancel"));
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				cancelButtonActionPerformed(evt);
			}
		});

		jPanel3.add(cancelButton);

		jPanel1.add(jPanel3, java.awt.BorderLayout.EAST);

		getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);

		jPanel4.setLayout(new java.awt.GridBagLayout());

		jPanel4.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(30, 70, 30, 70)));
		jLabel1.setText(UIManager.formatedHtmlString(ui.getString("fina2.description")));
		jLabel1.setFont(ui.getFont());
		jPanel4.add(jLabel1, UIManager.getGridBagConstraints(0, 0, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		descriptionText.setColumns(20);
		descriptionText.setFont(ui.getFont());
		jPanel4.add(descriptionText, UIManager.getGridBagConstraints(1, 0, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		jLabel2.setText(ui.getString("fina2.type"));
		jLabel2.setFont(ui.getFont());
		jPanel4.add(jLabel2, UIManager.getGridBagConstraints(0, 1, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		typeList.setFont(ui.getFont());
		typeList.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				typeListActionPerformed(evt);
			}
		});
		jPanel4.add(typeList, UIManager.getGridBagConstraints(1, 1, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		jLabel3.setText(UIManager.formatedHtmlString(ui.getString("fina2.ui.menu.action")));
		jLabel3.setFont(ui.getFont());
		jPanel4.add(jLabel3, UIManager.getGridBagConstraints(0, 2, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		jLabel4.setText(UIManager.formatedHtmlString(ui.getString("fina2.ui.menu.application")));
		jLabel4.setFont(ui.getFont());
		jPanel4.add(jLabel4, UIManager.getGridBagConstraints(0, 3, 10, 10, -1, -1, GridBagConstraints.EAST, -1, null));

		applicationText.setColumns(20);
		applicationText.setFont(ui.getFont());
		jPanel4.add(applicationText, UIManager.getGridBagConstraints(1, 3, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		actionText.setEditable(false);
		actionText.setColumns(20);
		actionText.setFont(ui.getFont());
		jPanel4.add(actionText, UIManager.getGridBagConstraints(1, 2, -1, -1, -1, -1, GridBagConstraints.WEST, -1, null));

		lookupActionButton.setIcon(ui.getIcon("fina2.lookup"));
		lookupActionButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				lookupActionButtonActionPerformed(evt);
			}
		});
		jPanel4.add(lookupActionButton, UIManager.getGridBagConstraints(2, 2, -1, -1, -1, -1, -1, -1, new java.awt.Insets(0, 5, 0, 0)));

		getContentPane().add(jPanel4, java.awt.BorderLayout.CENTER);

		pack();
	}// GEN-END:initComponents

	private void lookupActionButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:
		// event_lookupActionButtonActionPerformed
		SelectActionDialog dialog = new SelectActionDialog((java.awt.Frame) getParent(), true);

		dialog.show();

		TableRow row = dialog.getTableRow();

		if (row == null)
			return;

		actionKey = (String) row.getPrimaryKey();
		actionText.setText((String) ui.getAllAction(actionKey).getValue(javax.swing.AbstractAction.NAME));
	}// GEN-LAST:event_lookupActionButtonActionPerformed

	private void typeListActionPerformed(java.awt.event.ActionEvent evt) {// GEN-
		// FIRST
		// :
		// event_typeListActionPerformed
		switch (typeList.getSelectedIndex()) {
		case 0:
			actionText.setEnabled(true);
			lookupActionButton.setEnabled(true);
			applicationText.setEditable(false);
			break;
		case 1:
			actionText.setEnabled(false);
			lookupActionButton.setEnabled(false);
			applicationText.setEditable(true);
			break;
		}
	}// GEN-LAST:event_typeListActionPerformed

	private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN
		// -
		// FIRST
		// :
		// event_cancelButtonActionPerformed
		dispose();
	}// GEN-LAST:event_cancelButtonActionPerformed

	private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {

		if (!ui.isValidLength(descriptionText.getText(), true)) {

			ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.web.banktype.description.length"));
			return;
		} else if (descriptionText.getText().trim().length() == 0) {
			ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.pleaseEnterDescription"));
			return;
		}
		if (typeList.getSelectedIndex() == 0 && actionText.getText().length() == 0) {
			ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.menu.action"));
			return;
		} else if (typeList.getSelectedIndex() == 1 && applicationText.getText().trim().length() == 0) {
			ui.showMessageBox(null, ui.getString("fina2.title"), ui.getString("fina2.menu.application"));
			return;
		}

		if ((typeList.getSelectedIndex() == 0 && actionKey.trim() != "") || (typeList.getSelectedIndex() == 1)) {

			try {
				InitialContext jndi = fina2.Main.getJndiContext();

				Object ref = jndi.lookup("fina2/ui/menu/Menu");
				MenuHome home = (MenuHome) PortableRemoteObject.narrow(ref, MenuHome.class);

				try {
					if (node == null) {
						MenuPK parentPK = (MenuPK) parent.getPrimaryKey();
						menu = home.create(parentPK);
						// menu.setType(MenuConstants.MENU_ACTION_TYPE);
					}
				} catch (Exception e) {
					if (node == null)
						menu = null;
					Main.generalErrorHandler(e);
					return;
				}
				UserTransaction trans = main.getUserTransaction(jndi);
				trans.begin();
				try {
					menu.setDescription(main.getLanguageHandle(), descriptionText.getText());
					switch (typeList.getSelectedIndex()) {
					case 0:
						menu.setType(MenuConstants.MENU_ACTION_TYPE);
						menu.setActionKey(actionKey); // actionText.getText());
						break;
					case 1:
						menu.setType(MenuConstants.MENU_APPLICATION_TYPE);
						menu.setApplication(applicationText.getText());
						break;
					}
					trans.commit();
				} catch (Exception e) {
					Main.generalErrorHandler(e);
					trans.rollback();
					if (node == null) {
						menu.remove();
						menu = null;
					}
					return;
				}
				if (node == null) {
					node = new Node(menu.getPrimaryKey(), menu.getDescription(main.getLanguageHandle()), new Integer(menu.getType()));
				} else {
					node.setLabel(menu.getDescription(main.getLanguageHandle()));
				}
			} catch (Exception e) {
				Main.generalErrorHandler(e);
			}
			dispose();
			// }
		}

	}// GEN-LAST:event_okButtonActionPerformed

	/** Closes the dialog */
	private void closeDialog(java.awt.event.WindowEvent evt) {// GEN-FIRST:
		// event_closeDialog
		setVisible(false);
		dispose();
	}// GEN-LAST:event_closeDialog

	// Variables declaration - do not modify//GEN-BEGIN:variables
	private javax.swing.JPanel jPanel1;
	private javax.swing.JPanel jPanel2;
	private javax.swing.JButton helpButton;
	private javax.swing.JPanel jPanel3;
	private javax.swing.JButton okButton;
	private javax.swing.JButton cancelButton;
	private javax.swing.JPanel jPanel4;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JTextField descriptionText;
	private javax.swing.JLabel jLabel2;
	private javax.swing.JComboBox typeList;
	private javax.swing.JLabel jLabel3;
	private javax.swing.JLabel jLabel4;
	private javax.swing.JTextField applicationText;
	private javax.swing.JTextField actionText;
	private javax.swing.JButton lookupActionButton;
	// End of variables declaration//GEN-END:variables
}
